(*
 *  beware IRC services, bconfig.pas
 *  Copyright (C) 2002 Bas Steendijk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)
unit bconfig;

interface

uses boptiontable,pgtypes;
procedure onoperchannelchange(tag:taddrint;oldvalue:string);
type
  tcfglineproc=procedure(sv,sn:string);

const
  maxoptiontable=16;

var
  opt:record
    connectaddr:string;
    connectport:string;
    connectpass:string;
    myservername:string;
    info:string;
    myservernum:integer;
    botuserid:string;
    bothost:string;
    randseed:string;

    runasservice:boolean;
    servicename:string;
    signalport:string;
    signalpipe:string;

    autosave:integer;
    autosavenotice:boolean;
    operchannel:string;
    accountchars:string;
  end;

  optiontable:array[0..maxoptiontable] of Toptionentry=(
  (
  p:@opt.connectaddr;
  typ:opt_str;
  name:'ConnectAddr';
  info:'address of server to connect to';
  def_s:'127.0.0.1'),
  (
  p:@opt.connectport;
  typ:opt_str;
  name:'ConnectPort';
  info:'port of server to connect to';
  def_s:'4400'),
  (
  p:@opt.connectpass;
  typ:opt_str;
  name:'ConnectPass';
  info:'send password for link';
  def_s:'pass'),
  (
  p:@opt.myservername;
  typ:opt_str;
  name:'ServerName';
  info:'name of this services server (must be networkwide unique)';
  def_s:'services.server.name'),
  (
  p:@opt.info;
  typ:opt_str;
  name:'ServerInfo';
  info:'realname field of this server';
  def_s:'beware IRC services'),
  (
  p:@opt.myservernum;
  typ:opt_int;
  name:'ServerNum';
  info:'numeric of this server, must be networkwide unique and in the range 0-4095.';
  def_i:3),
  (
  p:@opt.botuserid;
  typ:opt_str;
  name:'BotUserid';
  info:'default userid for service bots';
  def_s:'service'),
  (
  p:@opt.bothost;
  typ:opt_str;
  name:'BotHost';
  info:'host field for service bots';
  def_s:'network.name'),
  (
  p:@opt.runasservice;
  typ:opt_bool;
  name:'NTService';
  info:'program runs as NT service (set using bserv install/uninstall, not manually) (windows only)';
  def_b:false),
  (
  p:@opt.servicename;
  typ:opt_str;
  name:'NTServiceName';
  info:'name of service in services manager (windows only)';
  def_s:'BewareServ'),
  (
  p:@opt.randseed;
  typ:opt_str;
  name:'RandSeed';
  info:'fill in a nonsense number or string, as extra seed (security) for the random function';
  def_s:'12345678'),
  (
  p:@opt.signalport;
  typ:opt_str;
  name:'SignalPort';
  info:'UDP port on 127.0.0.1 for restart/stop signals (windows only)';
  def_s:'45454'),
  (
  p:@opt.signalpipe;
  typ:opt_str;
  name:'SignalPipe';
  info:'named pipe for restart/stop signals (unix only)';
  def_s:'signalpipe'),
  (
  p:@opt.autosave;
  typ:opt_int;
  name:'AutoSave';
  info:'auto save database every nn seconds';
  def_i:1800),
  (
  p:@opt.autosavenotice;
  typ:opt_bool;
  name:'AutoSavenotice';
  info:'show notice for autosave';
  def_b:true),

  (
  p:@opt.operchannel;
  typ:opt_str;
  name:'operchannel';
  info:'event log channel';
  onchange:onoperchannelchange;
  def_s:'#Admin';

  ),
  (
  p:@opt.accountchars;
  typ:opt_str;
  name:'AccountChars';
  info:'allowed characters in account name, for first char, for other chars. A means A-Z, etc';

  def_s:'Aa0 Aa0';

  )


  );

  filespath:string;

procedure init;
procedure getpath;
procedure writecfg;

procedure readcfg(const filename:string;proc:tcfglineproc);

implementation

uses
  bcrash      , bstuff  , sysutils     , bparse , buser , readtxt , bircrandom ,
  bservicemngr, bconsts , replymanager ;
procedure onoperchannelchange(tag:taddrint;oldvalue:string);
var
  a:integer;
  s:tserviceunit;
begin
  //ipcheck_remoteclient(us);
  for a := maxserviceunit downto 0 do begin
    s := serviceunitarray[a];
    if s <> nil then if @s.onoperchannelchange <> nil then s.onoperchannelchange(oldvalue,s.tag);
  end;
end;

const
  cfgfile='bewareserv.ini';
  cfgtag='[bewareserv]';

procedure getpath;
var
  s:string;
  a,b:integer;
begin
  s := paramstr(0);
  b := 0;
  for a := length(s) downto 1 do begin
    if s[a] = '\' then begin
      b := a;
      break;
    end;
  end;
  filespath := copy(s,1,b);
  chdir(filespath);
end;

procedure init;
begin
  optiontable_default(optiontable,maxoptiontable);
  optiontable_load(optiontable,maxoptiontable,cfgfile,cfgtag);
  reply_default; // temporary until load/save of reply strings is done
  reply_load;

  {initialize things}
  setircrandomseed(opt.randseed);
  ircrandom;
end;



procedure readcfg(const filename:string;proc:tcfglineproc);
var
  sv,sn:string;
  a:integer;
  pt:treadtext;
  s:string;
begin
  assert(@proc <> nil,'readcfg: proc is not assigned','');
  if not readtext_init(pt,filename) then exit;
  while not readtext_eof(pt) do begin
    s := readtext_line(pt);
    if s <> '' then if s[1] <> ';' then begin
      while (length(s) > 1) and (s[length(s)] = ' ') do s := copy(s,1,length(s)-1);
      a := pos('=',s);
      if a <> 0 then begin
        sv := ircupper(copy(s,1,a-1));
        sn := copy(s,a+1,500);
        proc(sv,sn);
      end else begin
        proc(ircupper(s),'');
      end;
    end;
  end;
  readtext_close(pt);
end;

procedure writecfg;
begin
  optiontable_save(optiontable,maxoptiontable,cfgfile,cfgtag);
  reply_save;
end;


end.
