(*
 *  beware ircd, Internet Relay Chat server, bipcheck.pas
 *  Copyright (C) 2002 Bas Steendijk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)

unit bipcheck;

{
"IPcheck" does connection throttling, counting clones, and free targets

based on the idea of ircu's IPcheck.c (and designed to do exactly the same)

}

interface

uses blinklist,btime,bcmds,bstuff,bconsts,binipstuff;

const
  ipcheck_tablesize=$1000;

type
  tipcheck=class(tlinklist)
    ip:tbinip;
    online:integer;
  end;

var
  ipchecktable:array[0..ipcheck_tablesize-1] of tipcheck;

function ipcheck_hash(const ip:tbinip):integer;
function ipcheck_find(hash:integer;const ip:tbinip):tipcheck;
procedure ipcheck_remoteclient(p:pointer);
procedure ipcheck_disconnect(ipc:tipcheck);

procedure ipcheck_destroyall;

implementation

uses buser,bserver,bconfig;

function ipcheck_hash(const ip:tbinip):integer;
{$ifdef ipv6}
var
  a:integer;
{$endif}
begin
  {$ifdef ipv6}
  if ip.family = AF_INET6 then begin
    a := ip.ip6.s6_addr32[0] xor ip.ip6.s6_addr32[1] xor ip.ip6.s6_addr32[2] xor ip.ip6.s6_addr32[3];
    result := (a xor (a shr 16)) and (ipcheck_tablesize-1);
  end else
  {$endif}
  begin
    result := (ip.ip xor (ip.ip shr 16)) and (ipcheck_tablesize-1);
  end;
end;

function ipcheck_find(hash:integer;const ip:tbinip):tipcheck;
var
  ipc:tipcheck;
begin
  result := nil;
  ipc := ipchecktable[hash];
  while ipc <> nil do begin
    if comparebinip(ipc.ip,ip) then begin
      result := ipc;
      exit
    end;
    ipc := tipcheck(ipc.next);
  end;
end;

procedure ipcheck_remoteclient(p:pointer);
var
  us:tuser absolute p;
  hash:integer;
  ipc:tipcheck;
begin
  hash := ipcheck_hash(us.binip);
  ipc := ipcheck_find(hash,us.binip);

  if ipc = nil then begin
    ipc := tipcheck.create;
    ipc.ip := us.binip;
    linklistadd(tlinklist(ipchecktable[hash]),tlinklist(ipc));
  end;
  us.ipcheck := ipc;
  inc(us.ipcheck.online);
end;

{one client}
procedure ipcheck_disconnect(ipc:tipcheck);
begin
  if ipc = nil then exit;
  dec(ipc.online);
  if ipc.online <= 0 then begin
    linklistdel(tlinklist(ipchecktable[ipcheck_hash(ipc.ip)]),tlinklist(ipc));
    ipc.destroy;
  end;
end;

procedure ipcheck_destroyall;
var
  a:integer;
  p,p2:tipcheck;
begin
  for a := ipcheck_tablesize-1 downto 0 do begin
    p := ipchecktable[a];
    while p <> nil do begin
      p2 := tipcheck(p.next);
      linklistdel(tlinklist(ipchecktable[a]),tlinklist(p));
      p.destroy;
      p := p2;
    end;
  end;
  fillchar(ipchecktable,sizeof(ipchecktable),0);
end;

begin
  fillchar(ipchecktable,sizeof(ipchecktable),0);
end.
